@@ -162,7 +162,7 @@ pub mod tests {
         batches[0].packets[1].meta.size = shred.payload.len();
 
         let rv = verifier.verify_batches(batches);
-        assert!(!rv[0].packets[0].meta.discard);
-        assert!(rv[0].packets[1].meta.discard);
+        assert!(!rv[0].packets[0].meta.discard());
+        assert!(rv[0].packets[1].meta.discard());
     }
 }
